package com.schbrain.archetype.initializer.schedule;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNode;
import cn.hutool.core.lang.tree.TreeUtil;
import lombok.extern.slf4j.Slf4j;
import org.gitlab4j.api.GitLabApi;
import org.gitlab4j.api.GroupApi;
import org.gitlab4j.api.models.Group;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author liaozan
 * @since 2022/4/28
 */
@Slf4j
@Component
public class GitlabGroupFetchTask {

    @Autowired
    private GitLabApi gitLabApi;

    private List<Tree<Long>> groupTree = Collections.emptyList();

    public List<Tree<Long>> getGroupTree() {
        return groupTree;
    }

    @Scheduled(fixedRate = 1, timeUnit = TimeUnit.HOURS)
    public void fetchGroups() {
        try {
            GroupApi groupApi = gitLabApi.getGroupApi();
            List<Group> groups = groupApi.getGroups();
            List<TreeNode<Long>> treeNodes = groups.stream()
                    .map(group -> {
                        TreeNode<Long> treeNode = new TreeNode<>();
                        treeNode.setId(group.getId());
                        treeNode.setName(group.getName());
                        treeNode.setParentId(group.getParentId());
                        return treeNode;
                    }).collect(Collectors.toList());
            this.groupTree = TreeUtil.build(treeNodes, null);
        } catch (Exception e) {
            log.warn("Failed to fetch Gitlab groups", e);
        }
    }

}